/**
 ******************************************************************************
 *
 * @file       uavobjecttemplate.cpp
 * @author     The OpenPilot Team, http://www.openpilot.org Copyright (C) 2010.
 * @brief      Template for an uavobject in java
 *             This is a autogenerated file!! Do not modify and expect a result.
 *             Contains the calibration settings for the @ref AHRSCommsModule
 *
 * @see        The GNU Public License (GPL) Version 3
 *
 *****************************************************************************/
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

package org.openpilot.uavtalk.uavobjects;

import java.nio.ByteBuffer;
import java.util.ArrayList;
import java.util.List;
import java.util.ListIterator;

import org.openpilot.uavtalk.UAVObjectManager;
import org.openpilot.uavtalk.UAVObject;
import org.openpilot.uavtalk.UAVDataObject;
import org.openpilot.uavtalk.UAVObjectField;

/**
Contains the calibration settings for the @ref AHRSCommsModule

generated from ahrscalibration.xml
 **/
public class AHRSCalibration extends UAVDataObject {

	public AHRSCalibration() {
		super(OBJID, ISSINGLEINST, ISSETTINGS, NAME);
		
		List<UAVObjectField> fields = new ArrayList<UAVObjectField>();
		

		List<String> measure_varElemNames = new ArrayList<String>();
		measure_varElemNames.add("0");
		List<String> measure_varEnumOptions = new ArrayList<String>();
		measure_varEnumOptions.add("SET");
		measure_varEnumOptions.add("MEASURE");
		fields.add( new UAVObjectField("measure_var", "", UAVObjectField.FieldType.ENUM, measure_varElemNames, measure_varEnumOptions) );

		List<String> accel_biasElemNames = new ArrayList<String>();
		accel_biasElemNames.add("X");
		accel_biasElemNames.add("Y");
		accel_biasElemNames.add("Z");
		fields.add( new UAVObjectField("accel_bias", "m/s^2", UAVObjectField.FieldType.FLOAT32, accel_biasElemNames, null) );

		List<String> accel_scaleElemNames = new ArrayList<String>();
		accel_scaleElemNames.add("X");
		accel_scaleElemNames.add("Y");
		accel_scaleElemNames.add("Z");
		fields.add( new UAVObjectField("accel_scale", "(m/s^2)/lsb", UAVObjectField.FieldType.FLOAT32, accel_scaleElemNames, null) );

		List<String> accel_varElemNames = new ArrayList<String>();
		accel_varElemNames.add("X");
		accel_varElemNames.add("Y");
		accel_varElemNames.add("Z");
		fields.add( new UAVObjectField("accel_var", "(m/s^2)^2", UAVObjectField.FieldType.FLOAT32, accel_varElemNames, null) );

		List<String> gyro_biasElemNames = new ArrayList<String>();
		gyro_biasElemNames.add("X");
		gyro_biasElemNames.add("Y");
		gyro_biasElemNames.add("Z");
		fields.add( new UAVObjectField("gyro_bias", "rad/s", UAVObjectField.FieldType.FLOAT32, gyro_biasElemNames, null) );

		List<String> gyro_scaleElemNames = new ArrayList<String>();
		gyro_scaleElemNames.add("X");
		gyro_scaleElemNames.add("Y");
		gyro_scaleElemNames.add("Z");
		fields.add( new UAVObjectField("gyro_scale", "(rad/s)/lsb", UAVObjectField.FieldType.FLOAT32, gyro_scaleElemNames, null) );

		List<String> gyro_varElemNames = new ArrayList<String>();
		gyro_varElemNames.add("X");
		gyro_varElemNames.add("Y");
		gyro_varElemNames.add("Z");
		fields.add( new UAVObjectField("gyro_var", "(rad/s)^2", UAVObjectField.FieldType.FLOAT32, gyro_varElemNames, null) );

		List<String> gyro_tempcompfactorElemNames = new ArrayList<String>();
		gyro_tempcompfactorElemNames.add("X");
		gyro_tempcompfactorElemNames.add("Y");
		gyro_tempcompfactorElemNames.add("Z");
		fields.add( new UAVObjectField("gyro_tempcompfactor", "raw/raw", UAVObjectField.FieldType.FLOAT32, gyro_tempcompfactorElemNames, null) );

		List<String> mag_biasElemNames = new ArrayList<String>();
		mag_biasElemNames.add("X");
		mag_biasElemNames.add("Y");
		mag_biasElemNames.add("Z");
		fields.add( new UAVObjectField("mag_bias", "mGau", UAVObjectField.FieldType.FLOAT32, mag_biasElemNames, null) );

		List<String> mag_scaleElemNames = new ArrayList<String>();
		mag_scaleElemNames.add("X");
		mag_scaleElemNames.add("Y");
		mag_scaleElemNames.add("Z");
		fields.add( new UAVObjectField("mag_scale", "(mGau)/lsb", UAVObjectField.FieldType.FLOAT32, mag_scaleElemNames, null) );

		List<String> mag_varElemNames = new ArrayList<String>();
		mag_varElemNames.add("X");
		mag_varElemNames.add("Y");
		mag_varElemNames.add("Z");
		fields.add( new UAVObjectField("mag_var", "mGau^2", UAVObjectField.FieldType.FLOAT32, mag_varElemNames, null) );

		List<String> vel_varElemNames = new ArrayList<String>();
		vel_varElemNames.add("0");
		fields.add( new UAVObjectField("vel_var", "(m/s)^2", UAVObjectField.FieldType.FLOAT32, vel_varElemNames, null) );

		List<String> pos_varElemNames = new ArrayList<String>();
		pos_varElemNames.add("0");
		fields.add( new UAVObjectField("pos_var", "m^2", UAVObjectField.FieldType.FLOAT32, pos_varElemNames, null) );


		// Compute the number of bytes for this object
		int numBytes = 0;
		ListIterator<UAVObjectField> li = fields.listIterator();
		while(li.hasNext()) {
			numBytes += li.next().getNumBytes();
		}
		NUMBYTES = numBytes;

		// Initialize object
		initializeFields(fields, ByteBuffer.allocate(NUMBYTES), NUMBYTES);
		// Set the default field values
		setDefaultFieldValues();
		// Set the object description
		setDescription(DESCRIPTION);
	}

	/**
	 * Create a Metadata object filled with default values for this object
	 * @return Metadata object with default values
	 */
	public Metadata getDefaultMetadata() {
		UAVObject.Metadata metadata = new UAVObject.Metadata();
		metadata.gcsAccess = UAVObject.AccessMode.ACCESS_READWRITE;
		metadata.gcsTelemetryAcked = UAVObject.Acked.TRUE;
		metadata.gcsTelemetryUpdateMode = UAVObject.UpdateMode.UPDATEMODE_ONCHANGE;
		metadata.gcsTelemetryUpdatePeriod = 0;

		metadata.flightAccess = UAVObject.AccessMode.ACCESS_READWRITE;
		metadata.flightTelemetryAcked = UAVObject.Acked.TRUE;
		metadata.flightTelemetryUpdateMode = UAVObject.UpdateMode.UPDATEMODE_ONCHANGE;
		metadata.flightTelemetryUpdatePeriod = 0;

		metadata.loggingUpdateMode = UAVObject.UpdateMode.UPDATEMODE_NEVER;
		metadata.loggingUpdatePeriod = 0;
		return metadata;
	}

	/**
	 * Initialize object fields with the default values.
	 * If a default value is not specified the object fields
	 * will be initialized to zero.
	 */
	public void setDefaultFieldValues()
	{
		getField("measure_var").setValue("SET");
		getField("accel_bias").setValue(-73.5,0);
		getField("accel_bias").setValue(-73.5,1);
		getField("accel_bias").setValue(73.5,2);
		getField("accel_scale").setValue(0.0359,0);
		getField("accel_scale").setValue(0.0359,1);
		getField("accel_scale").setValue(-0.0359,2);
		getField("accel_var").setValue(0.0005,0);
		getField("accel_var").setValue(0.0005,1);
		getField("accel_var").setValue(0.0005,2);
		getField("gyro_bias").setValue(23,0);
		getField("gyro_bias").setValue(-23,1);
		getField("gyro_bias").setValue(23,2);
		getField("gyro_scale").setValue(-0.017,0);
		getField("gyro_scale").setValue(0.017,1);
		getField("gyro_scale").setValue(-0.017,2);
		getField("gyro_var").setValue(0.0001,0);
		getField("gyro_var").setValue(0.0001,1);
		getField("gyro_var").setValue(0.0001,2);
		getField("gyro_tempcompfactor").setValue(0,0);
		getField("gyro_tempcompfactor").setValue(0,1);
		getField("gyro_tempcompfactor").setValue(0,2);
		getField("mag_bias").setValue(0,0);
		getField("mag_bias").setValue(0,1);
		getField("mag_bias").setValue(0,2);
		getField("mag_scale").setValue(-2,0);
		getField("mag_scale").setValue(-2,1);
		getField("mag_scale").setValue(-2,2);
		getField("mag_var").setValue(50,0);
		getField("mag_var").setValue(50,1);
		getField("mag_var").setValue(50,2);
		getField("vel_var").setValue(0.4);
		getField("pos_var").setValue(0.4);

	}

	/**
	 * Create a clone of this object, a new instance ID must be specified.
	 * Do not use this function directly to create new instances, the
	 * UAVObjectManager should be used instead.
	 */
	public UAVDataObject clone(int instID) {
		// TODO: Need to get specific instance to clone
		try {
			AHRSCalibration obj = new AHRSCalibration();
			obj.initialize(instID, this.getMetaObject());
			return obj;
		} catch  (Exception e) {
			return null;
		}
	}

	/**
	 * Static function to retrieve an instance of the object.
	 */
	public AHRSCalibration GetInstance(UAVObjectManager objMngr, int instID)
	{
	    return (AHRSCalibration)(objMngr.getObject(AHRSCalibration.OBJID, instID));
	}

	// Constants
	protected static final int OBJID = 0x30101BB2;
	protected static final String NAME = "AHRSCalibration";
	protected static String DESCRIPTION = "Contains the calibration settings for the @ref AHRSCommsModule";
	protected static final boolean ISSINGLEINST = 1 == 1;
	protected static final boolean ISSETTINGS = 1 == 1;
	protected static int NUMBYTES = 0;


}